program detect_with_unit;

{*
 * Authors: Koen Van der Goten 
 *    Koen Van der Goten & Andre Vandierendonck
 *    Department of Experimental Psychology
 *    University of Ghent
 *    Henri Dunantlaan 2
 *    B-9000 Gent
 *    Phone: +32-(0)9-264 64 06 or +32-(0)9-264 64 37
 *    Fax: +32-(0)9-264 64 96
 *    E-mail: Koen.VanderGoten@rug.ac.be
 *            Andre.Vandierendonck@rug.ac.be
 * Program translated from C by Hans Vandierendonck
 * $Author: andre $
 * $Date: 1996/07/24 16:28:39 $
 * $Revision: 1.0 $
 * $Log: detectun.pas,v $
 * Revision 1.0  1996/07/24 16:28:39  andre
 * Initial revision
 *
 *
 * This program calculates indices of sensitivity and bias based
 * on several theories. All the functions used are part of a library
 * module, which is in a separate file.
 *
 *}  

uses
   dtctunit;

var
   n, ns, h1, f1, ns2, n2, h2, f2 : extended;
   extreme, extreme_ok : boolean;
   selection : selection_type;
   ch : char;
   str : string;

procedure printhead( ns, n, h1, f1, ns2, n2, h2, f2 : extended;
		     selection : selection_type; extreme : boolean );
   var
      junk : integer;

   begin
      junk := 0;
      if ( selection <> S_ROC ) then
      begin
	 writeln;
	 writeln('Hit ratio         : ', h1:3:0, ' / ', ns:3:0 );
	 writeln('False alarm ratio : ', f1:3:0, ' / ', n:3:0 )
      end
      else                      { ROC }
      begin
	 writeln;
	 writeln('Hit ratio         : ', h1:3:0, ' / ', ns:3:0,
		     ', ', h2:3:0, ' / ', ns2:3:0 );
	 writeln('False alarm ratio : ', f1:3:0, ' / ', n:3:0,
		     ', ', f2:3:0, ' / ', n2:3:0 )
      end;
      if extreme = FALSE then
      begin
	 if ( h1 = 0.0 ) and ( f1 = 0.0 ) and ( h1 = ns ) and ( f1 = n ) then
	 	junk := junk + 1;
	 if ( selection = S_ROC ) then
	 begin
	    if ( h2 = 0.0 ) and ( f2 = 0.0 ) and ( h2 = ns2 )
	    and ( f2 = n2 ) then
		   junk := junk + 1;
	 end;
	 if ( junk > 0 ) then
	 begin
	    writeln;
	    writeln('2N-Adjustment has been done for ratio 0.0 or of 1.0')
	 end
      end
      else	{ extreme }
      begin
	 writeln;
	 writeln('Adjustment with log-linear rule has been done')
      end
   end;

procedure gaussian( ns, n, h1, f1 : extended; extreme : boolean );

   var
      dprime, c, cprime, beta_g, logbeta_g : extended;

   begin
      dprime    := sens_g( ns, n, h1, f1, extreme );
      c         := loc_g( ns, n, h1, f1, extreme );
      cprime    := relloc_g( ns, n, h1, f1, extreme );
      beta_g    := like_g( ns, n, h1, f1, extreme );
      logbeta_g := loglike_g( ns, n, h1, f1, extreme );

      writeln;
      writeln('Gaussian Distribution');
      writeln('d''           = ', dprime:11:6 );
      writeln('c            = ', c:11:6 );
      if ( cprime = HUGE ) then
      begin
	 writeln('c''           =    undefined');
	 writeln('Beta(G)      =    undefined');
	 writeln('log(Beta(G)) =    undefined')
      end
      else
      begin
	 writeln('c''           = ', cprime:11:6 );
	 writeln('Beta(G)      = ', beta_g:11:6 );
	 writeln('log(Beta(G)) = ', logbeta_g:11:6 )
      end
   end;

procedure logistic( ns, n, h1, f1 : extended; extreme : boolean );

   var
   	logalfa, alfa, b, beta_l, logbeta_l : extended;
	logb, bprime, bprime_hodos, bsecundo : extended;

   begin
      logalfa      := sens_l( ns, n, h1, f1, extreme );
      logb         := logloc_l( ns, n, h1, f1, extreme );
      alfa         := sens_alfa( ns, n, h1, f1, extreme );
      b            := loc_l( ns, n, h1, f1, extreme );
      beta_l       := like_l( ns, n, h1, f1, extreme );
      logbeta_l    := loglike_l( ns, n, h1, f1, extreme );
      bprime       := relloc_l( ns, n, h1, f1, extreme );
      bprime_hodos := hodos( ns, n, h1, f1, extreme );
      bsecundo     := grier( ns, n, h1, f1, extreme );

      writeln;
      writeln('Logistic Distribution');
      writeln('logalfa      = ', logalfa:11:6 );
      writeln('log(b)       = ', logb:11:6 );
      writeln('alfa         = ', alfa:11:6 );
      writeln('b            = ', b:11:6 );
      if ( bprime = HUGE ) then
      begin
	 writeln('b''           =   undefined');
	 writeln('Beta(L)      =   undefined');
	 writeln('log(Beta(L)) =   undefined');
	 writeln('B''(H)        =   undefined');
	 writeln('B''''          =   undefined')
      end
      else
      begin
	 writeln('b''           = ', bprime:11:6 );
	 writeln('Beta(L)      = ', beta_l:11:6 );
	 writeln('log(Beta(L)) = ', logbeta_l:11:6 );
	 writeln('B''(H)        = ', bprime_hodos:11:6 );
	 writeln('B''''          = ', bsecundo:11:6 )
      end
   end;

procedure rectangular( ns, n, h1, f1 : extended ; extreme : boolean );

   var
   	q, p_c, k, kprime : extended;

   begin
      q        := sens_q( ns, n, h1, f1, extreme );
      p_c      := sens_pc( ns, n, h1, f1, extreme );
      k        := loc_k( ns, n, h1, f1, extreme );
      kprime   := relloc_k( ns, n, h1, f1, extreme );

      writeln;
      writeln('Rectangular Distributions');
      writeln('q            = ', q:11:6 );
      writeln('p(c)         = ', p_c:11:6 );
      writeln('k            = ', k:11:6 );
      writeln('k''           = ', kprime:11:6 )
   end;

procedure other( ns, n, h1, f1 : extended ; extreme : boolean );

   var
   	lor, yule, aprime : extended;

   begin
      lor      := sens_lor( ns, n, h1, f1, extreme );
      yule     := sens_yule( ns, n, h1, f1, extreme );
      aprime   := sens_np( ns, n, h1, f1, extreme );

      writeln;
      writeln('`Nonparametric''');
      writeln('A''           = ', aprime:11:6 );
      writeln;
      writeln('Sensitivity measures proposed by Swets (1986)');
      writeln('LOR          = ', lor:11:6 );
      writeln('Yule''s Q     = ', yule:11:6 )
   end;

procedure roc( ns, n, h1, f1, ns2, n2, h2, f2 : extended ; extreme : boolean );

   var
      s, deltam, da, de, az : extended;
   
   begin
      s      := roc_slope( ns, n, h1, f1, ns2, n2, h2, f2, extreme );
      deltam := delta_m( ns, n, h1, f1, ns2, n2, h2, f2, extreme );
      da     := roc_da( ns, n, h1, f1, ns2, n2, h2, f2, extreme );
      de     := roc_de( ns, n, h1, f1, ns2, n2, h2, f2, extreme );
      az     := roc_az( ns, n, h1, f1, ns2, n2, h2, f2, extreme );

      writeln;
      writeln('ROC measures');
      writeln('slope        = ', s:11:6 );
      writeln('delta m      = ', deltam:11:6 );
      writeln('da           = ', da:11:6 );
      writeln('d''e          = ', de:11:6 );
      writeln('Az           = ', az:11:6 )
   end;

begin
   detect_ver;

   repeat
      writeln('	- determine which calculations will be performed:');
      writeln('	  + G:   gives calculations under GAUSSIAN assumptions');
      writeln('	  + L:   gives calculations under LOGISTIC assumptions');
      writeln('	  + R:   gives calculations under RECTANGULAR assumptions');
      writeln('	  + O:   gives OTHER than previous calculations');
      writeln('	  + A:   gives ALL preceding calculations');
      writeln('	  + ROC: gives calculations calculations based on ROC (2 points)');
      write('	? ');
      readln( str );
      if ( str = 'G' ) or ( str = 'g' ) then
	  selection := S_GAUSSIAN
      else if ( str = 'L' ) or ( str = 'l' ) then
	  selection := S_LOGISTIC
      else if ( str = 'R' ) or ( str = 'r' ) then
	  selection := S_RECTANGULAR
      else if ( str = 'O' ) or ( str = 'o' ) then
	  selection := S_OTHER
      else if ( str = 'A' ) or ( str = 'a' ) then
	  selection := S_ALL
      else if ( str = 'ROC' ) or ( str = 'roc' ) then
	  selection := S_ROC
      else
          selection := S_ILLEGAL
   until ( selection <> S_ILLEGAL );

   repeat
   begin
      extreme_ok := FALSE;
      writeln('	- set flag for calculation with extreme proportions:');
      writeln('	  + 2N:  correction with 2N');
      writeln('	  + LL:  log-linear rule');
      write('	? ');
      readln( str );
      if ( str = '2N' ) or ( str = '2n' ) then
      begin
         extreme := FALSE;
         extreme_ok := TRUE
      end
      else if ( str = 'LL' ) or ( str = 'll' ) then
      begin
         extreme := TRUE;
         extreme_ok := TRUE
      end
   end
   until ( extreme_ok = TRUE );

   write('	- enter number of signal presentations for the first point: ');
   readln( ns );
   write('	- enter number of noise presentations for the first point: ');
   readln( n );
   write('	- enter number of hits for the first point: ');
   readln( h1 );
   write('	- enter number of false alarms for the first point: ');
   readln( f1 );

   if ( selection = S_ROC ) then
   begin
   write('	- enter number of signal presentations for the second point: ');
   readln( ns2 );
   write('	- enter number of noise presentations for the second point: ');
   readln( n2 );
      write('	- enter number of hits for the second point: ');
      readln( h2 );
      write('	- enter number of false alarms for the second point: ');
      readln( f2 )
   end;

   printhead( ns, n, h1, f1, ns2, n2, h2, f2, selection, extreme );

   case selection of
      S_GAUSSIAN: gaussian( ns, n, h1, f1, extreme );
      S_LOGISTIC: logistic( ns, n, h1, f1, extreme );
      S_RECTANGULAR: rectangular( ns, n, h1, f1, extreme );
      S_OTHER: other( ns, n, h1, f1, extreme );
      S_ROC: roc( ns, n, h1, f1, ns2, n2, h2, f2, extreme );
      S_ALL:
	 begin
	    gaussian( ns, n, h1, f1, extreme );
	    logistic( ns, n, h1, f1, extreme );
	    rectangular( ns, n, h1, f1, extreme );
	    other( ns, n, h1, f1, extreme );
	 end;
   end;

end.
